/******************************************************************************

 @file  tof_security.h

 @brief This file contains methods to enable/disable and control TOF
 Group: WCS, BTS
 Target Device: cc13x2_26x2

 ******************************************************************************
 
 Copyright (c) 2018-2019, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************
 
 
 *****************************************************************************/

/**
 *  @defgroup TOF TOF
 *  @brief This module implements the Time of Flight (TOF)
 *
 *  @{
 *  @file       tof_security.h
 *  @brief      ToF security module interface
 */

#ifndef TOF_SECURITY_H_
#define TOF_SECURITY_H_

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * INCLUDES
 */

#include <ti/drivers/AESCTRDRBG.h>

#include <stdint.h>

/*********************************************************************
 * MACROS
 */
/// @brief NUM_ELEMENTS calculates the num of elements in an array
#define NUM_ELEMENTS(x)                 (sizeof(x)/sizeof(x[0]))

/*********************************************************************
 * CONSTANTS
 */
#define TOF_SEC_FILL_BOTH_BUFFS         0x01  //!< Fill both buffers
#define TOF_SEC_FILL_1ST_BUFF           0x02  //!< Fill first buffers
#define TOF_SEC_FILL_2ND_BUFF           0x03  //!< Fill second buffers

#define TOF_SEC_SYNCWORD_SIZE           8     //!< Length of Syncword

#define TOF_SEC_DBL_BUFF_SIZE           16    //!< Size of ToF_securoty double buffer

/*********************************************************************
 * TYPEDEFS
 */

/** @defgroup TOF_SEC_Structs TOF Security Structures
 * @{
 */

/// @brief ToF Security Mode
typedef enum
{
  TOF_MODE_SINGLE_BUF = 0,     //!< Single Buffer
  TOF_MODE_DBL_BUF             //!< Double Buffer
} tofSecMode_e;

/// @brief ToF Security Error Result
typedef enum 
{
  TOF_SEC_SUCCESS_E = 0,       //!< Success
  TOF_SEC_GEN_FAIL_E,          //!< Seed Generation fail
  TOF_SEC_AES_FAIL_E,          //!< AES fail
  TOF_SEC_ALLOC_FAIL_E,        //!< Allocation fail
  TOF_SEC_INVALID_PRMS_FAIL_E  //!< Invalid parameters fail
} tofSecErrRes_t;

/// @brief ToF Security Configuration Parameters
typedef struct
{
  uint8_t             bUseDoubleBuffer;           //!< Defaults to false
  uint16_t            syncWordSize;               //!< In bytes. Defaults to 8
  uint16_t            totalNumOfSyncWords;        //!< Defaults to 64
} tofSecCfgPrms_t;

/// @brief ToF Security Handle
typedef struct
{
  tofSecCfgPrms_t     tofSecCfgParams;             //!< ToF Security Configuration Parameters

  // Sync Words
  uint16_t            numOfSyncWordsPerBuffer;     //!< Length of double buffer, both buffers must be equal length. The number of bursts in double buffer mode is still numSyncwordsPerBurst
  uint8_t             *pSyncWordBuffer1;           //!< Pointer to the sync word buffer, used in both single and double buffer mode
  uint8_t             *pSyncWordBuffer2;           //!< Pointer to the second sync word buffer, used in double buffer mode only
  uint8_t             *pTempSyncWordBuffer;        //!< Pointer to a temporary buffer used to hold a buffer in case sync is lost

  // DRBG
  AESCTRDRBG_Handle   drbgHandle;                  //!< DRBG handle

  // Seed
  uint8_t             seed[AESCTRDRBG_SEED_LENGTH_AES_128];      //!< ctr DRBG Seed
} tofSecHandle_t;

/** @} End TOF_SEC_Structs */

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/**
 * @brief       This function to initialize the tof security parameters.
 *
 * input parameters
 *
 * @param       tofSecCfgPrms  - A pointer to the user-defined tof security preset values.
 *
 * @param       tofSecHandle  - A pointer to an empty tof security handle.
 *
 * output parameters
 *
 * @param       tofSecHandle  - A pointer to the tof security state.
 *
 * @return      int - NULL if failed, other if succeeded
 */
int TOFSecurity_paramsInit(tofSecHandle_t *tofSecHandle, tofSecCfgPrms_t *tofSecCfgPrms);

/**
 * @brief       This function is used to open AESCTRDRBG driver and, when needed,
 *              to generate 128-bit random seed.
 *
 * input parameters
 *
 * @param       tofSecHandle  - A pointer to an empty tof security handle.
 *
 * @param       seed - NULL if need to generate seed, otherwise, a pointer to 128-bit random seed.
 *
 * output parameters
 *
 * @param       tofSecHandle  - A pointer to the tof security state.
 *
 * @return      int - NULL if failed, other if succeeded.
 */
int TOFSecurity_open(tofSecHandle_t *tofSecHandle, uint8_t *seed);
/**
 * @brief       This function is used to close ToF Security and free buffers
 *
 * input parameters
 *
 * @param       tofSecHandle  - A pointer to an empty tof security handle.
 *
 * output parameters
 *
 * @return      int - NULL if failed, other if succeeded
 */
int TOFSecurity_close(tofSecHandle_t *tofSecHandle);

/**
* @brief    Will fill each Sync Word Buffer with numOfSyncWordsPerBuffer Sync Words
*
* input parameters
*
* @param   bufferToFill  - An enum describing which buffers to fill
*
* @param   tofSecHandle  - A pointer to the tof security state.
*
* output parameters
*
* @param   tofSecHandle  - A pointer to the tof security state.
*
* @return  NULL if failed, other if succeeded
*/
int TOFSecurity_genSyncWords(tofSecHandle_t *tofSecHandle, uint8_t bufferToFill);

/*********************************************************************
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif // TOF_SECURITY_H_

/** @} End TOF */
